<?php

namespace Lean;

use League\Route\Strategy;

use League\Route\Http\Exception\NotFoundException;
use Lean\Factory\HttpFactoryInterface;
use Picea\Asset\Asset;
use Psr\Container\ContainerInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Picea\Picea;
use Psr\Http\Server\RequestHandlerInterface;
use function DI\get;

class ApplicationStrategy extends Strategy\ApplicationStrategy {

    public const ASSET_TRIGGER_UPDATE = [
        "js", "mjs", "manifest", "webmanifest", "css", "png", "ico",
        "jpg", "jpeg", "gif", "webp", "woff", "woff2", "eot", "svg",
        "ttf"
    ];

    public function __construct(
        protected Picea $picea,
        protected ContainerInterface $di,
        protected HttpFactoryInterface $httpFactory,
    ) {}

    public function getNotFoundDecorator(NotFoundException $exception): MiddlewareInterface
    {
        return new class($this->picea, $this->di, $this->httpFactory) implements MiddlewareInterface {

            public function __construct(
                protected Picea $picea,
                protected ContainerInterface $di,
                protected HttpFactoryInterface $httpFactory,
            ) { }

            public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
            {
                if (php_sapi_name() !== 'cli' and ! defined('STDIN'))  {
                    return $this->throw404($request);
                }

                return $handler->handle($request);;
            }

            public function throw404(ServerRequestInterface $request) : ResponseInterface
            {
                if ( getenv('DEBUG') && $this->di->has(\Picea\Asset\Asset::class) ) {
                    $params = $request->getServerParams();

                    $scpName = basename(explode('?', $params['REQUEST_URI'] ?? "", 2)[0]);
                    list(, $ext) = array_pad(explode('.', $scpName), 2, null);

                    if ($ext && in_array($ext, ApplicationStrategy::ASSET_TRIGGER_UPDATE)) {
                        $this->di->get(\Picea\Asset\Asset::class)->launchInstall();
                    }
                }

                return $this->httpFactory->createHtmlResponse($this->picea->renderHtml("lean/error/404", [], $this), 404);
            }
        };
    }
}